==> GeneticsBase_1.0.0.tar.gz: GeneticsBase/R/alleleCount.R <==
## Count of number of alleles matching value ##


## This implementation is simpler, but dramatically slower..
## allele.count.geneSet <- function(object,
##                                  value,
##                                  marker=markerNames(object))
##   {
##     if(missing(value))
##       value <- alleleLevels(object)
##     else if(!is.list(value))
##         {
##           value <- rep(list(value), length=length(marker))
##           names(value) <- marker
##         }
## 
##     sapply(marker,
##            function(m)
##            apply(extractAlleles(object, which=c(1,2), marker=m),
##                  1,
##                  function(x) sum(x==value[[m]]))
##            )
##   }
## 

##
## This one is much faster
##
alleleCount.geneSet <- function(
                                object,
                                value=alleleLevels(object, marker),
                                marker=markerNames(object)
                                )
  {
    if(!is.character(marker))
       marker <- markerNames(object)[marker]
    
    if(!is.list(value))
        {
          value <- rep(list(value), length=length(marker))
          names(value) <- marker
        }

    retval <- matrix(
                     as.numeric(NA),
                     nrow=nobs(object),
                     ncol=length(unlist(value))
                     )

    rownames(retval) <- rownames(object@sampleInfo)
    #if(max(sapply(value, length))>1)
    colnames(retval) <-unlist(lapply( marker,
                                     function(M)
                                     paste(M, value[[M]], sep=".")
                                     ))
    #else
    #  colnames(retval) <- marker
       

    for(m in marker)
      {
        cc <- callCodes(object,m)
        tt <- transTables(object,m)[,2:3]

        alleles <- value[[m]]
        
        for(a in alleles)
          {
            ct <- apply(tt, 1, function(x) sum(x==a,na.rm=T))
            ac <- ct[cc]
            #if(length(alleles)==1)
            #  retval[,m] <- ac
            #else
            retval[,paste(m,a,sep='.')] <- ac
          }
      }

    retval

  }

setGeneric("alleleCount", function(object, ...)
           standardGeneric("alleleCount"))
        
setMethod("alleleCount", "geneSet", alleleCount.geneSet)

==> GeneticsBase_1.14.0.tar.gz: GeneticsBase/R/alleleCount.R <==
## Count of number of alleles matching value ##


## This implementation is simpler, but dramatically slower..
## allele.count.geneSet <- function(object,
##                                  value,
##                                  marker=markerNames(object))
##   {
##     if(missing(value))
##       value <- alleleLevels(object)
##     else if(!is.list(value))
##         {
##           value <- rep(list(value), length=length(marker))
##           names(value) <- marker
##         }
## 
##     sapply(marker,
##            function(m)
##            apply(extractAlleles(object, which=c(1,2), marker=m),
##                  1,
##                  function(x) sum(x==value[[m]]))
##            )
##   }
## 

##
## This one is much faster
##
alleleCount.geneSet <- function(
                                object,
                                value=alleleLevels(object, marker),
                                marker=markerNames(object)
                                )
  {
    if(!is.character(marker))
       marker <- markerNames(object)[marker]
    
    if(!is.list(value))
        {
          value <- rep(list(value), length=length(marker))
          names(value) <- marker
        }

    retval <- matrix(
                     as.numeric(NA),
                     nrow=nobs(object),
                     ncol=length(unlist(value))
                     )

    rownames(retval) <- rownames(object@sampleInfo)
    #if(max(sapply(value, length))>1)
    colnames(retval) <-unlist(lapply( marker,
                                     function(M)
                                     paste(M, value[[M]], sep=".")
                                     ))
    #else
    #  colnames(retval) <- marker
       
    #browser()
    for(m in marker)
      {
        cc <- callCodes(object,m)
        tt <- transTables(object,m)##[,2:3]
        alleles <- value[[m]]
        na.allele <- which(apply(tt[,-1,drop=FALSE],1,function(x)all(is.na(x))))
        na.allele <- ifelse(length(na.allele), tt[na.allele,1],"none")
        tt <- tt[,-1,drop=FALSE]
        #else na.allele <- "none"
        for(a in alleles)
          {
            if (is.na(na.allele))              
              if (is.na(a))ct <- apply(tt, 1, function(x)sum(is.na(x)))
              else ct <- apply(tt, 1, function(x)sum(x==a,na.rm=TRUE))
            else{
              if(a==na.allele)
               ct <- apply(tt, 1, function(x)sum(is.na(x)))
              else ct <- apply(tt, 1, function(x)sum(x==a,na.rm=TRUE))
            }
            ##if(any(is.na(x)))sum(is.na(x)) else sum(x==a,na.rm=TRUE))
            ac <- ct[cc]
            #if(length(alleles)==1)
            #  retval[,m] <- ac
            #else
            retval[,paste(m,a,sep='.')] <- ac
          }
      }

    retval

  }

setGeneric("alleleCount", function(object, ...)
           standardGeneric("alleleCount"))
        
setMethod("alleleCount", "geneSet", alleleCount.geneSet)
